/*
 * Created on October 26, 2005, 4:58 PM
 */

package com.sunlabs.archsim.fleet.fleets.fibonacci;


import java.util.List;
import java.util.Iterator;

import com.sunlabs.archsim.runner.Runner;
import com.sunlabs.archsim.runner.OperationLoader;
import com.sunlabs.archsim.base.hierarchy.Entities;
import com.sunlabs.archsim.engine.Director;

// import com.sunlabs.archsim.fleet.HornAndFunnnel.SourceFunnel;

import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.fleet.ships.testing.DataSource;
import com.sunlabs.archsim.fleet.ships.testing.PayloadSink;
import com.sunlabs.archsim.fleet.ships.testing.PayloadSource;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

/**
 *
 * @author mh193489
 */
public class FibonacciTest extends TestCase {
    
    private static String fileName= "src/" + FibonacciTest.class.getPackage().getName().replace('.', '/') + "/FibonacciScript.xml";
    
    private static Director director;
    
    /* References to components to be tested */
    private static PayloadSink dataSink;
    
    private static final String DATA1_SOURCE= "DataSourceA";
    private static final String DATA2_SOURCE= "DataSourceB";
    private static final String DATA3_SOURCE= "DataSourceC";
    private static final String DATA4_SOURCE= "DataSourceD";
    
    private static final String PAYLOAD_SOURCE= "InstrGen";
    
    /* port numbers */
    
    private static final int DATASOURCE_A_OUT=   0;
    private static final int ADDER_OUT=          1;
    private static final int ACCUMULATOR_OUT=    2;
    private static final int FIFO_A_OUT=         3;
    private static final int FIFO_B_OUT=         4;
    private static final int DUPLICATOR_A_OUT1=   5;
    private static final int DUPLICATOR_B_OUT1=   6;
    private static final int DATASOURCE_B_OUT=   7;
    private static final int DATASOURCE_C_OUT=   8;
    private static final int DATASOURCE_D_OUT=   9;
    private static final int SELECTOR_OUT=      10;
    private static final int DUPLICATOR_A_OUT2= 11;
    private static final int DUPLICATOR_B_OUT2= 12;
    
    private static final int DATASINK_IN=         0;
    private static final int ADDER_IN1=           1;
    private static final int ADDER_IN2=           2;
    private static final int ACCUMULATOR_IN=      3;
    private static final int FIFO_A_IN=           4;
    private static final int FIFO_B_IN=           5;
    private static final int DUPLICATOR_A_IN=     6;
    private static final int DUPLICATOR_B_IN=     7;
    private static final int SELECTOR_IN1=        8;
    private static final int SELECTOR_IN2=        9;
    private static final int SELECTOR_IN_SELECT= 10;
//    private static final int DATASINK_B_IN=          11;
//    private static final int DATASINK_C_IN=          12;
    
    /* (N+2)th fibonacci number to accumulate */
    private static final int N= 8;
    
//    private static SourceFunnel sourceFunnel;
    
    
    protected void setUp() { 
        // reset sources from previous runs
        DataSource.clear();
        PayloadSource.clear();
         
        // for sum test
        DataSource.add(DATA1_SOURCE, new Data(1));
        
        DataSource.add(DATA2_SOURCE, new Data(1));
        
        for (int i= 0; i < N; i++) {
            DataSource.add(DATA3_SOURCE, new Data(1));
        }
        
        DataSource.add(DATA3_SOURCE, new Data(2));
        DataSource.add(DATA3_SOURCE, new Data(false));
        
//        PayloadSource.add(DATA4_SOURCE, new Data(false));
        DataSource.add(DATA4_SOURCE, new Data(false));
        
        
        /* move instructions running the FLEET */
        
        /* initialize fibonacci sequence with 1 and 1
         * and the accumulator with the first two fibonacci numbers
         */
        move(DATASOURCE_A_OUT,  DUPLICATOR_A_IN);
        move(DUPLICATOR_A_OUT1,  FIFO_A_IN);
        move(DUPLICATOR_A_OUT2,  ACCUMULATOR_IN);
        
        move(DATASOURCE_B_OUT,  DUPLICATOR_B_IN);
        move(DUPLICATOR_B_OUT1,  FIFO_B_IN);
        move(DUPLICATOR_B_OUT2,  ACCUMULATOR_IN);
        
        
        /* compute the fibonacci sequence and accumulate it
         *
         * Note that step N+1 needed to feed the Accumulator SHIP with
         * OOB token to finalize the sequence.
         */
        
        for (int i= 0; i<N+1; i++) {
            move(FIFO_A_OUT,  DUPLICATOR_A_IN);
            move(DUPLICATOR_A_OUT1,  ADDER_IN1);
            move(DUPLICATOR_A_OUT2,  FIFO_B_IN);
            move(FIFO_B_OUT,  ADDER_IN2);
            move(ADDER_OUT,  SELECTOR_IN1);
            move(DATASOURCE_C_OUT, SELECTOR_IN_SELECT);
            move(SELECTOR_OUT, DUPLICATOR_B_IN);
            move(DUPLICATOR_B_OUT1,  FIFO_A_IN);
            move(DUPLICATOR_B_OUT2, ACCUMULATOR_IN );
        }
        
        move(DATASOURCE_D_OUT, SELECTOR_IN2);
        
        /* move the final result to datasink */
        move(ACCUMULATOR_OUT, DATASINK_IN);
        
        move(FIFO_A_OUT,  DUPLICATOR_A_IN);
        move(DUPLICATOR_A_OUT1,  ADDER_IN1);
        move(DUPLICATOR_A_OUT2,  FIFO_B_IN);
        move(FIFO_B_OUT,  ADDER_IN2);
        move(ADDER_OUT,  SELECTOR_IN1);

        move(DATASOURCE_C_OUT, SELECTOR_IN_SELECT);

        move(FIFO_B_OUT,  ADDER_IN2);

        
        // initialize the tests
        director= Director.getInstance();
        
        
        OperationLoader ldr= new OperationLoader();
        List ops= ldr.load(fileName);
        
        Runner r= new Runner();
        
        r.perform(ops);
        
        director.journal("Finished Test run");
        
        // get references to the components
        
        dataSink = (PayloadSink)Entities.find("DataSink");
    }
    
    protected void tearDown() {
        // clean up after tests
        //director.closeJournal();
        director.masterClear();
    }
    
    public static Test suite() {
        return new TestSuite(FibonacciTest.class);
    }
    
    
    private static void move(int source, int destination) {
        Payload payload= new Payload();
        payload.source= source;
        payload.destination= destination;
        PayloadSource.add(PAYLOAD_SOURCE, payload);
    }
    
    public static void main(String[] args) {
        
        // run the tests
        junit.textui.TestRunner.run(FibonacciTest.class);
        

    } // main
    
    public void testTestSetup() {
        // data sink
        assertNotNull(dataSink);
        assertNotNull(dataSink.receivedData);
        for (Iterator<Payload> i= dataSink.receivedData.iterator(); i.hasNext();) {
            // check if data is not null
            Data data= i.next().data;
            assertNotNull(data);
            // check if data value is not null, in case data is valid
            if (data.valid) {
                assertNotNull(data.value);
            }
        }
        
    } 
    
    /* Tests for the data interface */
    
    public void testAccumulatedSum() {
        assertTrue(dataSink.receivedData.elementAt(0).data.valid);
        assertEquals( ((Integer)dataSink.receivedData.elementAt(0).data.value).intValue(), 143);
    }
    
    
}
