/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

/*
 * Created on Feb 7, 2005
 *
 */
package com.sunlabs.archsim.tests.splitter;

import com.sunlabs.archsim.gasp.State;

/**
 * @author ib27688
 *
 * A factory to generate splitter groups.
 * The cycle is the number of outputs the splitter will produce
 */
public class SplitterFactory {
	
	private SplitterFactory() {}
		
	public static Splitter please(String name, int cycle) {
		Splitter splitter= new Splitter(name);
		
		// make the states
		State dataInState= new State(SplitterGlobal.DATA_IN_STATE_NAME);
		dataInState.setParent(splitter);		
		State ackDataInState= new State(SplitterGlobal.ACK_DATA_IN_STATE_NAME);
		ackDataInState.setParent(splitter);		
		State dataOutState= new State(SplitterGlobal.DATA_OUT_STATE_NAME);
		dataOutState.setParent(splitter);		
		State ackDataOutState= new State(SplitterGlobal.ACK_DATA_OUT_STATE_NAME);
		ackDataOutState.setParent(splitter);		
		
		// set appropriate states for the splitter
		splitter.setDataIn(dataInState);
		splitter.setAckDataIn(ackDataInState);
		splitter.setDataOut(dataOutState);
		splitter.setAckDataOut(ackDataOutState);

		State remainderState= null;
		
		for (int i= 0; i< SplitterGlobal.cycle; i++ ) {
			// make terminals
			SplitterInput dataInTerminal= new SplitterInput(
					SplitterGlobal.DATA_IN_TERMINAL_NAME);
			dataInTerminal.setParent(splitter);
			SplitterOutput remainderOutTerminal= new SplitterOutput(
					SplitterGlobal.REMAINDER_OUT_TERMINAL_NAME);
			remainderOutTerminal.setParent(splitter);
			SplitterOutput splitOutTerminal= new SplitterOutput(
					SplitterGlobal.SPLIT_BITS_OUT_TERMINAL_NAME);
			splitOutTerminal.setParent(splitter);
			SplitterInput ackSplitOutTerminal= new SplitterInput(
					SplitterGlobal.ACK_SPLIT_BITS_TERMINAL_NAME);
			ackSplitOutTerminal.setParent(splitter);
			
			//attach output terminals to appropriate states
			dataOutState.attach(splitOutTerminal);
			ackDataOutState.attach(ackSplitOutTerminal);
			
			// make the action
			SplitterAction action= new SplitterAction(SplitterGlobal.SPLIT_ACTION_NAME + "_" + i);
			action.setParent(splitter);
			
			// attach the terminals to the action
			action.attach(dataInTerminal);
			action.attach(remainderOutTerminal);
			action.attach(splitOutTerminal);
			action.attach(ackSplitOutTerminal);
			
			// make connections here to use the previous state
			// then make a new state and make approproate connections			
			
			if (i==0) {
				dataInState.attach(dataInTerminal);
			}
			else  {
				// here the remainder state from the previous iteration is used
				remainderState.attach(dataInTerminal);
			}			
			
			// make the next state unless this is the last iteration
			if (i < SplitterGlobal.cycle - 1) {
				remainderState= new State(SplitterGlobal.REMAINDER_STATE_NAME + "_" + i);
				remainderState.setParent(splitter);
				remainderState.attach(remainderOutTerminal);
			}
			else {
				ackDataInState.attach(remainderOutTerminal);
			}
		
		} // for
		return splitter;
	} // please

}
